extensions [r gis csv nw py]
globals [time-unit time-now current-hour current-day exit-trucks exited-this-tick current-hour-enter-trucks station-results truck-results summary-data]

breed [ODAgents ODAgent] ;;this agent to store origin and possible destination
breed [FinalDestinations FinalDestination] ;;the exit destination
breed [trucks truck] ;;truck agent
breed [ProfileAgents ProfileAgent]
breed [EntryNumAgents EntryNumAgent]
breed [nodes node]
breed [stations station]  ;;create gasstation agent
breed [newStations newStation]
breed [connectors connector] ;;create connector agent
;breed [enternodes enternode] ;;create enter nodes
stations-own [;;station agent attribute
  station-ID
  station-name
  station-location
  station-type
  station-capacity
  station-serviceclass
  station-customers
  station-violations
  station-usage-ratio

  average-hourly ;for processing file
  currentHourUse
]
newStations-own[
  newStation-ID
  newStation-type
]
connectors-own[ ;;connector attributes
  connector-ID]
ODAgents-own[
  origin ;;origin link
  destination ;;possible destination
  factor-60day ;;
  destination-ratio ;;
  external-link;; if 0 means there is going to be next stop, if 1 means end of journey
]
FinalDestinations-own[
  finaldestination-ID
  finaldestiantion-ratio
]
ProfileAgents-own[
  profile-type ;; enteringtruck, existingtruck
  profile-ID
  Driving-Day
  NotDriving-Day
  OffDuty-Day
  last-rest-duration
  Driving-Since-Last-Rest
  enternode-origin
  entry-ratio
  entry-hour
  entry-weekday
  entry-status
]
EntryNumAgents-own[
  entry-type ;;existing or entering
  entry-node-ID
  entry-parking-ID ;;this exclusive for the trucks off-duty at parking stations initially
  entry-hour
  entry-num
]
trucks-own[
  truck-ID
  enternode-origin
  entry-ratio
  external-link?
  entry-hour ;;the hour the truck entered
  entry-weekday ;;the day the truck entered
  profile-type ;; enteringtruck or existingtruck

  node-destination ;;destination is setup through origin-destination lookup table
  my-route
  current-location-node
  distance-traveled

  ;last 7 days record
  Driving-Day
  NotDriving-Day
  OffDuty-Day


  ;;record current day summary
  current-day-driving-time
  current-day-not-driving-time
  current-day-off-duty-time

  status? ;; on duty driving, on duty not driving, off duty
  last-rest-duration ;;initial rest time past
  last-rest-duration ;;last rest duration indicating rest type
  Driving-Since-Last-Rest;;the driving time since last rest
  current-duty-total-driving-time  ;;the current duty driving time
  current-duty-total-not-driving-time ;;the off time the current duty
  on-duty-rest-time ;;the rest time since rest start
  off-duty-rest-time ;;rest time after 7 days on duty, max is 34 hours
  rest-time-left ;;rest time left at this break

  currentsteptime ;;record the leftover time from previous location

  time-to-destination ;;the time needed to destination
  rest-time-need ;; the time needed for the next stop
  risk-type ;; risk-taker or risk-averse
  parking-search-window ;;when to search for the next stop
  RDW ;;remaining driving window
  RDDW ;; remaining driving time in driving window
  RRH ;;remaining rolling hour
  R8H ;; remaining 8 hour on-duty time

  violation-parking ;;true if violating parking rules
  violation-HOS ;;true if violating HOS rules
]

nodes-own[node-link-ID first-node? last-node? node-location]
links-own [link-ID dismiles speed]


to startup
  user-message(word "Please click the \"INFO\" tab for instructions on setting up the simulation. Click OK or Halt to continue.")
end

;load shapefile into the interface
to Setup
  __clear-all-and-reset-ticks
  set time-now 0
  set current-day 0
  set current-hour 0
  set exit-trucks 0

  load-shapefile
  load-csv


  ;;existing trucks on duty
  ask EntryNumAgents with [entry-type = "existing-on-duty"][
    let num floor (entry-num)
    generate-existing-truck-agents 0 num entry-node-ID "existing-on-duty"]
  ;;existing trucks off duty
  ask EntryNumAgents with [entry-type = "existing-off-duty"][
    let num floor (entry-num)
    generate-existing-truck-agents 0 num entry-node-ID "existing-off-duty"]

  update-stations-connectors

  ;;time unit in hour
  set time-unit Tick-Minutes / 60
  set station-results []
  set truck-results []

  violation-update

  ;;create empty output files for results
  csv:to-file "./results/StationsResults.csv" []
  file-open "./results/StationsResults.csv"
  file-print csv:to-row (list "CurrentTick" "Name" "Parking ID" "Number of Parked Trucks" "Number of Violated Trucks" "Parking Capacity" "Ratio")
  file-flush
  file-close

  csv:to-file "./results/truckresultsweek0.csv" []
  file-open "./results/truckresultsweek0.csv"
  file-print csv:to-row (list "Truck ID" "On Duty Driving" "On Duty Not Driving" "Off Duty")
  file-flush
  file-close
  csv:to-file "./results/truckresultsweek1.csv" []
  file-open "./results/truckresultsweek1.csv"
  file-print csv:to-row (list "Truck ID" "Truck Type" "On Duty Driving" "On Duty Not Driving" "Off Duty")
  file-flush
  file-close
  csv:to-file "./results/truckresultsweek2.csv" []
  file-open "./results/truckresultsweek2.csv"
  file-print csv:to-row (list "Truck ID" "Truck Type" "On Duty Driving" "On Duty Not Driving" "Off Duty")
  file-flush
  file-close

  Export-Trucks-CSV-0 ;;updates week 0 csv file with historical profiles for existing trucks

end

to load-shapefile

 gis:set-world-envelope (list -94.617919 -89.644395 32.97306594661921 36.530640053380786)

 let ak gis:load-dataset "Ar.shp"
   foreach gis:feature-list-of ak
   [vector-feature ->
     gis:set-drawing-color grey
     gis:draw vector-feature 1.0
     ask patches gis:intersecting ak[set pcolor gray]]

  let hwy gis:load-dataset "InterstatesHighwaysAR.shp"
  let previous-node nobody
  foreach gis:feature-list-of hwy
  [vector-feature ->
    let linkid gis:property-value vector-feature "id"
    let dismile gis:property-value vector-feature "length"
    foreach (gis:vertex-lists-of vector-feature)[vertex ->
      let first-node nobody
      foreach vertex[point ->
        let location gis:location-of point
        if not empty? location[
          create-nodes 1[
            set hidden? true
            set node-location location
            set node-link-ID linkid
            setxy (item 0 location)(item 1 location)
            if first-node = nobody [set first-node self set first-node? true]
            if previous-node != nobody [
              create-link-with previous-node[
                set speed highway-speed
                set dismiles dismile
                set link-ID linkid
                set color white
            ]]
            set previous-node self
          ]
        ]
      ]
      ask node ([who] of previous-node)[set last-node? true]
      set previous-node nobody
    ]
  ]
  ;;link the nodes at common vertex
  ask nodes with [first-node? = true or last-node? = true][
    let lc-x item 0 node-location
    let lc-y item 1 node-location
    create-links-with other nodes-here with [(item 0 node-location = lc-x and item 1 node-location = lc-y) and (first-node? = true or last-node? = true)][
      set speed highway-speed
      set dismiles 0
      set color white]
  ]

  let connect gis:load-dataset "Connectors.shp"
  foreach gis:feature-list-of connect
  [vector-feature ->
    ask patches gis:intersecting vector-feature[sprout-connectors 1[

      set connector-ID gis:property-value vector-feature "iid"
      set hidden? true
      set color grey
      set shape "circle"
      set size 2]]]

   let pps gis:load-dataset "ExistingStations.shp"
   foreach gis:feature-list-of pps
   [vector-feature ->
    ask patches gis:intersecting vector-feature [
      sprout-stations 1 [

        set color green
        set station-ID gis:property-value vector-feature "IDNumber"
        set station-type gis:property-value vector-feature "FacilityTy"
        ifelse station-type = "Public" [set shape "parkingpublic"][set shape "parkingprivate"] ;;identify the station provide fast food as red
        set station-capacity gis:property-value vector-feature "Capacity"
        set station-serviceclass gis:property-value vector-feature "ServiceCla"
        set station-name gis:property-value vector-feature "name"
        ;set station-location gis:property-value vector-feature "highwaylocation"
        set station-usage-ratio 0
        set size 5]]
  ]

  gis:set-world-envelope (gis:envelope-union-of (gis:envelope-of hwy) (gis:envelope-of ak)(gis:envelope-of pps))
end
to load-csv
  file-open "EnteringTrucksDestinations.csv"
  let skip-first-row 0
  while [not file-at-end?][
    let odlist csv:from-row file-read-line
    if skip-first-row > 0[
      create-ODAgents 1[
        set origin item 0 odlist
        set destination item 1 odlist
        set factor-60day item 2 odlist
        set destination-ratio item 5 odlist
        set external-link item 4 odlist
        set hidden? true
      ]
    ]
    set skip-first-row 1
  ]
  file-close

  file-open "EnteringTrucksProfiles.csv"
  set skip-first-row 0
  while [not file-at-end?][
    let truckprofile csv:from-row file-read-line
    ;;
    if skip-first-row > 0 [
      create-ProfileAgents 1[
        set profile-type "enteringtruck"
        set profile-ID item 0 truckprofile
        let DD []
        set DD lput (item 1 truckprofile)  DD
        set DD lput item 2 truckprofile DD
        set DD lput item 3 truckprofile DD
        set DD lput item 4 truckprofile DD
        set DD lput item 5 truckprofile DD
        set DD lput item 6 truckprofile DD
        set DD lput item 7 truckprofile DD
        set Driving-Day DD
        let ND []
        set ND lput item 8 truckprofile ND
        set ND lput item 9 truckprofile ND
        set ND lput item 10 truckprofile ND
        set ND lput item 11 truckprofile ND
        set ND lput item 12 truckprofile ND
        set ND lput item 13 truckprofile ND
        set ND lput item 14 truckprofile ND
        set NotDriving-Day ND
        let OD []
        set OD lput item 15 truckprofile OD
        set OD lput item 16 truckprofile OD
        set OD lput item 17 truckprofile OD
        set OD lput item 18 truckprofile OD
        set OD lput item 19 truckprofile OD
        set OD lput item 20 truckprofile OD
        set OD lput item 21 truckprofile OD
        set OffDuty-Day OD

        ;;removed lines to round historic profiles daily totals to 24 hours

        set last-rest-duration item 22 truckprofile
        set Driving-Since-Last-Rest item 23 truckprofile
        set enternode-origin item 24 truckprofile
        set entry-ratio item 25 truckprofile
        set entry-hour item 26 truckprofile
        set entry-weekday item 27 truckprofile]]
    set skip-first-row skip-first-row + 1
  ]
  file-close

;;Existing truck profile
file-open "ExistingTrucksProfiles.csv"
  set skip-first-row 0
  while [not file-at-end?][
    let truckprofile csv:from-row file-read-line

    ;;
    if skip-first-row > 0 [
      create-ProfileAgents 1[
        set profile-type "existingtruck"
        set profile-ID item 0 truckprofile
        let DD []
        set DD lput (item 1 truckprofile) DD
        set DD lput item 2 truckprofile DD
        set DD lput item 3 truckprofile DD
        set DD lput item 4 truckprofile DD
        set DD lput item 5 truckprofile DD
        set DD lput item 6 truckprofile DD
        set DD lput item 7 truckprofile DD
        set Driving-Day DD
        ;print profile-ID
        ;print DD
        ;print Driving-Day
        let ND []
        set ND lput item 8 truckprofile ND
        set ND lput item 9 truckprofile ND
        set ND lput item 10 truckprofile ND
        set ND lput item 11 truckprofile ND
        set ND lput item 12 truckprofile ND
        set ND lput item 13 truckprofile ND
        set ND lput item 14 truckprofile ND
        set NotDriving-Day ND
        let OD []
        set OD lput item 15 truckprofile OD
        set OD lput item 16 truckprofile OD
        set OD lput item 17 truckprofile OD
        set OD lput item 18 truckprofile OD
        set OD lput item 19 truckprofile OD
        set OD lput item 20 truckprofile OD
        set OD lput item 21 truckprofile OD
        set OffDuty-Day OD

        ;;adujust driving+not driving+off-duty to 24 hours by equally add the gap
        ;REMOVED THE FOLLOWING LINES
;        let total-time (map + (map + Driving-Day NotDriving-Day) OffDuty-Day)
;        let gap map[x -> (24 - x) / 3] total-time
;        set Driving-Day (map + Driving-Day gap)
;        set NotDriving-Day (map + NotDriving-Day gap)
;        set OffDuty-Day (map + OffDuty-Day gap)

        set last-rest-duration item 22 truckprofile
        set Driving-Since-Last-Rest item 23 truckprofile
        set enternode-origin item 24 truckprofile
        set entry-ratio item 25 truckprofile
        set entry-hour item 26 truckprofile
        set entry-weekday item 27 truckprofile ;;0 Sunday, 1 Saturday ... 6 Monday ;;5 tuesday 4 wednesday 3 thursday 2 friday??????????????
        set entry-status item 28 truckprofile ;;1 on duty driving, 2 on duty not driving, 3 off duty
        ;;there is no on-duty-not-driving profile for existing trucks
        if entry-status = 2 [die]
    ]]
    set skip-first-row skip-first-row + 1
  ]
  file-close

;;put entry volume as agents
  file-open "TrucksEntryVolume.csv"
  set skip-first-row 0
  while [not file-at-end?][ ;;read full file
;    while [skip-first-row < 30][ ;;read top lines
    let entryprofile csv:from-row file-read-line
    ;;
    if skip-first-row > 0 [
      create-EntryNumAgents 1[
        set entry-type "entering"
        set entry-node-ID item 0 entryprofile
        set entry-hour item 1 entryprofile
        set entry-num item 2 entryprofile
      ]
    ]
    set skip-first-row skip-first-row + 1
  ]
  file-close

  ;;put existing on duty truck locations as agents
  file-open "ExistingOnDutyMidnightLocation.csv"
    set skip-first-row 0
  while [not file-at-end?][
;    while [skip-first-row < 30][
    let entryprofile csv:from-row file-read-line
    if skip-first-row > 0 [
      create-EntryNumAgents 1[
        set entry-type "existing-on-duty"
        set entry-node-ID item 0 entryprofile
        set entry-num int ((item 1 entryprofile))
    ]]
    set skip-first-row skip-first-row + 1]
  file-close

  ;;put existing off duty trucks locations as agents
  file-open "ExistingOffDutyMidnightLocation.csv"
    set skip-first-row 0
  while [not file-at-end?][
;    while [skip-first-row < 30][
    let entryprofile csv:from-row file-read-line
    if skip-first-row > 0 [
      create-EntryNumAgents 1[
        set entry-type "existing-off-duty"
        set entry-parking-ID item 0 entryprofile
        set entry-num int ((item 3 entryprofile))
        set entry-node-ID item 4 entryprofile
    ]]
    set skip-first-row skip-first-row + 1]
  file-close

  ;;put final destination as agents
  file-open "ExternalLinks.csv"
  set skip-first-row 0
  while[not file-at-end?][
    let finaldestinationfile csv:from-row file-read-line
    if skip-first-row > 0[
      create-FinalDestinations 1[
        set finaldestination-ID item 0 finaldestinationfile
        set finaldestiantion-ratio item 1 finaldestinationfile
      ]
    ]
    set skip-first-row skip-first-row + 1
  ]
  file-close
end

;;create entering trucks based on imported profiles
to generate-entering-truck-agents [entryhour num originnode]
  if any? ProfileAgents with [profile-type = "enteringtruck" and enternode-origin = originnode]
  [
    let risknum int (Risk-Taker-Ratio / 100 * num)
    hatch-trucks int num
    [
      set truck-ID who
      set enternode-origin originnode
      let driveday []
      let offdutyday []
      let notdriveday []
      let restduration 0
      let drivesincelastrest 0
      let entryratio 0
      let entryweekday 0
      set profile-type "enteringtruck"
      set rest-time-left (time-unit * 3)

      ask one-of ProfileAgents with [profile-type = "enteringtruck" and enternode-origin = originnode]
      [
        set driveday Driving-Day ;;ProfileAgent driving-day
        set offdutyday OffDuty-Day
        set notdriveday NotDriving-Day
        set restduration last-rest-duration
        set drivesincelastrest Driving-Since-Last-Rest
        set entryratio entry-ratio
        set entryweekday entry-weekday
      ]
      set Driving-Day driveday ;;truck Driving-Day
      set NotDriving-Day notdriveday
      set OffDuty-Day offdutyday
      set last-rest-duration restduration
      set Driving-Since-Last-Rest drivesincelastrest
      set entry-hour entryhour
      set entry-weekday entryweekday
      if one-of nodes with [node-link-ID = originnode] = nobody
        [show originnode]
      set current-location-node one-of nodes with [node-link-ID = originnode]
      move-to current-location-node
      set status? "On-Duty-Driving"

      set currentsteptime 0
      let p-type "enteringtruck"
      truck-find-path enternode-origin p-type

      ;;only certain percentage of trucks will take risk(risk takers wait longer to start searching for parking)
      ifelse risknum > 0
        [set risk-type "risk-taker"]
        [set risk-type "risk-averse"]
      set risknum risknum - 1

      set shape "truck"
      set color blue
      set size 6
      set hidden? False
    ]
  ]
  Export-Trucks-CSV-0
end
;;create existing trucks based on imported profiles
to generate-existing-truck-agents [entryhour num originnode ttype]
  let estatus 0
  ifelse ttype = "existing-on-duty"[set estatus 1][set estatus 3]
  let risknum int (Risk-Taker-Ratio / 100 * int (num ))
  if any? ProfileAgents with [profile-type = "existingtruck" and entry-status = estatus]
  [
    let status 0
    hatch-trucks int (num)
    [
      set truck-ID who
      set enternode-origin originnode
      let driveday []
      let offdutyday []
      let notdriveday []
      let restduration 0
      let drivesincelastrest 0
      let entryratio 0
      let entryweekday 0
      set profile-type "existingtruck"
      set rest-time-left (time-unit * 3)
      ;print truck-ID

      ask one-of ProfileAgents with [profile-type = "existingtruck" and entry-status = estatus]
      [
        if entry-status = 1 [set status "On-Duty-Driving"]
;        if entry-status = 2 [set status "On-Duty-Not-Driving"]
        if entry-status = 3 [set status "Off-Duty"]
        set driveday Driving-Day
        ;print driveday
        set offdutyday OffDuty-Day
        set notdriveday NotDriving-Day
        set restduration last-rest-duration
        set drivesincelastrest Driving-Since-Last-Rest
        set entryratio entry-ratio
        set entryweekday entry-weekday
      ]
      set Driving-Day driveday
      set NotDriving-Day notdriveday
      set OffDuty-Day offdutyday
      set last-rest-duration restduration
      set Driving-Since-Last-Rest drivesincelastrest
      set entry-hour entryhour
      set entry-weekday entryweekday
      if one-of nodes with [node-link-ID = originnode] = nobody
        [show originnode]
      set current-location-node one-of nodes with [node-link-ID = originnode]
      move-to current-location-node
      set status? status

      set currentsteptime 0
      let p-type "existingtruck"
      truck-find-path enternode-origin p-type

      ;;only certain percentage of truks will take risk
      ifelse risknum > 0
        [set risk-type "risk-taker"]
        [set risk-type "risk-averse"]
      set risknum risknum - 1

      ;setup the initial status
      ifelse status? = "On-Duty-Driving"
      [
        ifelse last-rest-duration <= 3
        [
          set RDW 14 - (item 5 NotDriving-Day + last-rest-duration + driving-since-last-rest)
          set RDDW DailyDrivingHours - driving-since-last-rest
        ]
        [
          set RDW 14 - (current-duty-total-not-driving-time + driving-since-last-rest)
          set RDDW DailyDrivingHours - driving-since-last-rest
        ]
      ]
      [ ;;if status is Off Duty do this (no existing trucks have the status On Duty Not Driving)

;        set RRH 60 - (sum Driving-Day + sum NotDriving-Day)
;        let randgama 8 + random-gamma 1 (1 / 4.806)
;        while [randgama > 24]
;          [set randgama 8 + random-gamma 1 (1 / 4.806)]
;        set rest-time-left max list (8 + random-gamma 1 (1 / 4.806) - last-rest-duration) 0

        ifelse last-rest-duration <= 0.75
        [

          let randgama 8 + random-gamma 1 (1 / 4.806)
          while [randgama > 24]
            [set randgama 8 + random-gamma 1 (1 / 4.806)]
          set rest-time-left max list (8 + random-gamma 1 (1 / 4.806) - last-rest-duration) 0
        ]
        [
          set RRH 60 - (sum Driving-Day + sum NotDriving-Day)
          set RDDW DailyDrivingHours - driving-since-last-rest
          let randgama 8 + random-gamma 1 (1 / 4.806)
          while [randgama > 24]
            [set randgama 8 + random-gamma 1 (1 / 4.806)]
          set rest-time-left max list (8 + random-gamma 1 (1 / 4.806) - last-rest-duration) 0
          set RDW 14 - (item 5 NotDriving-Day + driving-since-last-rest + last-rest-duration + rest-time-left)
        ]
      ]

      set shape "truck"
      set color 25
      set size 6
      set hidden? False
    ]
  ]

  Export-Trucks-CSV-0


end
to truck-find-path [ori p-type]
  let my-r current-location-node
  let destinationnode ""
  let e-link? ""
  let dest ""
  ;;use Roulette Wheel method to find the destination
  let rndwheel random-float 1

  carefully
  [
    ifelse p-type = "enteringtruck"
    [ ;;;if true do this
      ask min-one-of ODAgents with [origin = ori] [abs (destination-ratio - rndwheel)]
      [
  ;  ask one-of ODAgents with [origin = ori][
        set e-link? external-link
        set dest destination
      ]
    ]
    [ ;;; if false do this
      ask min-one-of ODAgents with [origin = ori] [abs (destination-ratio - rndwheel)]
       [
          set dest destination
          set e-link? external-link
       ]
    ]
   set node-destination dest
   set external-link? e-link?
  ][die]

  let dnodeid node-destination
  let timetodestination 0

  if one-of nodes with [node-link-ID = dnodeid] = nobody
    [show dnodeid]

  set destinationnode one-of nodes with [node-link-ID = dnodeid]

  ask current-location-node
    [set timetodestination nw:weighted-distance-to destinationnode dismiles
     set my-r nw:turtles-on-weighted-path-to destinationnode dismiles]

  if timetodestination = false
    [set timetodestination 0]

  set time-to-destination (timetodestination / Highway-Speed)

  ifelse my-r != false
    [set my-route my-r]
    [let mlist []
      set mlist lput current-location-node mlist
      set my-route mlist]
end

to start


    set exited-this-tick 0
    ask trucks
    [

     if status? = "On-Duty-Driving"
     [
       set current-day-driving-time current-day-driving-time + time-unit
       truck-move-to-destination
       set time-to-destination time-to-destination - time-unit
       set Driving-Since-Last-Rest Driving-Since-Last-Rest + time-unit
       set current-duty-total-driving-time current-duty-total-driving-time + time-unit
       set current-day-driving-time current-day-driving-time + time-unit

       ;; truck start to search next station for rest
       if (8 - Driving-Since-Last-Rest) <= parking-search-window
       [
         ;set violation-HOS true ;;violate HOS rules
         ifelse (8 - Driving-Since-Last-Rest) <= 0
         [ ;;if truck has driven more than 8hrs since last rest
           ;set status? "Off-Duty"
           set violation-HOS true
           set last-rest-duration rest-time-need
           set rest-time-left rest-time-need
           ifelse any? stations-here with [station-usage-ratio < 1][ ;if
             let havespace true
             set status? "Off-Duty"
             set last-rest-duration rest-time-need
             set rest-time-left rest-time-need]
            [ let havespace false ;else
            	;set status? "Off-Duty"
              ;ask stations-here [set station-violations station-violations + 1]
        ]]
         [ ;;if truck has not driven more than 8 hours since last rest, but has entered parking search window
           if any? stations-here
           [
            let havespace false
            ask stations-here
            [
               ifelse station-usage-ratio < 1
                [set havespace true]
                [set havespace false] ; set station-violations station-violations + 1]
            ]
            if havespace = true and rest-time-need > 0
            [
              set status? "Off-Duty"
              set last-rest-duration rest-time-need
              set rest-time-left rest-time-need
            ]
           ]
         ]
    ]]

       if status? = "On-Duty-Not-Driving"
       [
         set rest-time-left (rest-time-left - time-unit)
         set current-duty-total-not-driving-time current-duty-total-not-driving-time + time-unit
         set current-day-not-driving-time current-day-not-driving-time + time-unit
         if rest-time-left <= 0 [
           set status? "On-Duty-Driving"]
       ]

       if status? = "Off-Duty"
       [
         set off-duty-rest-time off-duty-rest-time + time-unit
         set rest-time-left rest-time-left - time-unit
         set current-day-off-duty-time current-day-off-duty-time + time-unit
         if rest-time-left <= 0 ;;start fresh after a long time off duty break
         [
           set status? "On-Duty-Driving"
           set Driving-Since-Last-Rest 0
           set last-rest-duration rest-time-need
           if last-rest-duration >= 8
           [ ;;new duty start
             set current-duty-total-driving-time 0
             set current-duty-total-not-driving-time 0
           ]
         ]
       ]
  ] ;;end of ask trucks block

  ;;stop station search strategy for entering trucks
  ask trucks with [profile-type = "enteringtruck"][
    ifelse last-rest-duration <= 8[
      set RDW 14 - max (list (current-day-driving-time + last-rest-duration + current-day-not-driving-time) (Driving-Since-Last-Rest + last-rest-duration + current-day-not-driving-time))
      set RDDW DailyDrivingHours - max(list current-day-driving-time driving-since-last-rest)
      set RRH 60 - (sum Driving-Day + sum NotDriving-Day)
      set R8H 8 - (driving-since-last-rest + current-duty-total-not-driving-time)]
    [set RDW 14 - (driving-since-last-rest + current-duty-total-not-driving-time)
      set RDDW DailyDrivingHours - driving-since-last-rest
      set RRH 60 - sum Driving-Day - sum NotDriving-Day
      set R8H 8 - (driving-since-last-rest + current-duty-total-not-driving-time)]
    let timing-list (list RDW RDDW RRH R8H)
    let timing-position position (min timing-list) timing-list
    if timing-position = 0 [set rest-time-need 8 + random-gamma 1 (1 / 4.806) while [rest-time-need > 24][set rest-time-need 8 + random-gamma 1 (1 / 4.806)]
      ifelse risk-type = "risk-taker"[set parking-search-window 1][set parking-search-window 3]]
    if timing-position = 1 [set rest-time-need 8 + random-gamma 1 (1 / 4.806) while [rest-time-need > 24][set rest-time-need 8 + random-gamma 1 (1 / 4.806)]
      ifelse risk-type = "risk-taker"[set parking-search-window 1][set parking-search-window 3]]
    if timing-position = 2 [set rest-time-need max (list 10 (24 - time-now + 3 + random 2))
      ifelse risk-type = "risk-taker"[set parking-search-window 1][set parking-search-window 3]]
    if timing-position = 3 [set rest-time-need random-gamma 1 0.570707
      ifelse risk-type = "risk-taker"[set parking-search-window 1][set parking-search-window 3]]]

  ask trucks with [profile-type = "existingtruck"][
    ;;setup the initial status
    ifelse status? = "On-Duty-Driving"[
      ifelse last-rest-duration <= 8[
        set RDW 14 - (item 5 NotDriving-Day + last-rest-duration + driving-since-last-rest)
        set RDDW DailyDrivingHours - driving-since-last-rest]
      [set RDW 14 - (current-duty-total-not-driving-time + driving-since-last-rest)
        set RDDW DailyDrivingHours - driving-since-last-rest]]
    [set RRH 60 - (sum Driving-Day + sum NotDriving-Day)
      set rest-time-left (random-weibull 0.03219412 1.07355099) - last-rest-duration]

    let timing-list (list RDW RDDW RRH R8H)
    let timing-position position (min timing-list) timing-list
    if timing-position = 0 [set rest-time-need 8 + random-gamma 1 (1 / 4.806) while [rest-time-need > 24][set rest-time-need 8 + random-gamma 1 (1 / 4.806)]
      ifelse risk-type = "risk-taker"[set parking-search-window 1 ][set parking-search-window 3]]
    if timing-position = 1 [set rest-time-need 8 + random-gamma 1 (1 / 4.806) while [rest-time-need > 24][set rest-time-need 8 + random-gamma 1 (1 / 4.806)]
      ifelse risk-type = "risk-taker"[set parking-search-window 1][set parking-search-window 3]]
    if timing-position = 2 [set rest-time-need max (list 10 (24 - time-now + 3 + random 2))
      ifelse risk-type = "risk-taker"[set parking-search-window 1][set parking-search-window 3]]
    if timing-position = 3 [set rest-time-need random-gamma 1 0.570707
      ifelse risk-type = "risk-taker"[set parking-search-window 1][set parking-search-window 3]]
  ]

  update-stations-connectors
  ask trucks[
    ifelse profile-type = "existingtruck"[set color orange][set color blue]
    if status? = "Off-Duty" [set color white]]

  ;;current time
  set current-day floor (time-now / 24)
  set current-hour floor (time-now) - current-day * 24

  ifelse current-day < Number-of-days-to-run-the-model
    [tick]
    [process-R
     ifelse Summarize-Usage-Data = true [post-process stop] [stop]]
  set time-now time-now + time-unit
  let new-hour floor (time-now) - current-day * 24
  let new-day floor (time-now / 24)

  ;;create trucks every hour
  if new-hour > current-hour [
    ask EntryNumAgents with [entry-hour = new-hour and entry-type = "entering"][
      set current-hour-enter-trucks int (entry-num)
      generate-entering-truck-agents current-hour current-hour-enter-trucks entry-node-ID]]

  if new-day > current-day[
    ask trucks[
      set Driving-Day remove-item 6 Driving-Day
      set NotDriving-Day remove-item 6 NotDriving-Day
      set OffDuty-Day remove-item 6 OffDuty-Day
      set Driving-Day fput current-day-driving-time Driving-Day
      set NotDriving-Day fput current-day-not-driving-time NotDriving-Day
      set OffDuty-Day fput current-day-off-duty-time OffDuty-Day
      set current-day-driving-time 0
      set current-day-not-driving-time 0
      set current-day-off-duty-time 0]
  ]

  if current-day > 6[
    if Save-Truck-Results = true [Export-Trucks-CSV-2]]
  if (current-day <= 6)[
    if Save-Truck-Results = true [Export-Trucks-CSV-1]]
;   if current-day = 6
;   [
;     if Save-Truck-Results = true [Export-Trucks-CSV-1]
;   ]
;   if current-day = 13
;   [
;     if Save-Truck-Results = true [Export-Trucks-CSV-2]
;   ]

  violation-update

  ;;save station results
  if Save-Stations-Usage-Results = true [export-stations-csv]

 ; print median [station-usage-ratio] of stations
;  let res count trucks with [status? = "Off-Duty"]
;  print res / 6887
  ;;model simulating two weeks
  if current-day > Number-of-days-to-run-the-model[process-R
     ifelse Summarize-Usage-Data = true
       [post-process stop] [stop]]

end
;;stop model
to stop-model
  stop
end
to truck-move-to-destination
;  carefully[
  let distonew 0
  let currentpositionnode current-location-node
  let currentposition position currentpositionnode my-route

  if length my-route > currentposition + 1[
    let new-location item (currentposition + 1) my-route

    while [currentsteptime < time-unit][  ;; currently assume truck need to move several vertex to get to 1 hour node
      set currentposition (position currentpositionnode my-route)
      ifelse currentposition < (length my-route - 1)[
        set new-location (item (currentposition + 1) my-route)
        face new-location
        ask currentpositionnode [
          let lk link-with new-location
          ask lk[set distonew dismiles]]
        set distance-traveled distance-traveled + distonew
        set currentsteptime currentsteptime + (distonew / Highway-Speed)
          move-to new-location
        set currentpositionnode new-location
        set current-location-node new-location

      ][stop]
    ]

    if currentsteptime >= time-unit[
      move-to new-location
      set current-location-node new-location
      set currentsteptime currentsteptime - time-unit]]

    if position current-location-node my-route >= length my-route - 1 [
    set status? "On-Duty-Not-Driving"
;    set rest-time-left random-weibull 0.003212287 0.386510718
      ifelse external-link? = 0[
;        let dest find-final-destination
;        truck-find-final-path dest
      set external-link? 1
      ;set color 25
      ] ;;set the truck as exiting mode
    [set exited-this-tick exited-this-tick + 1 set exit-trucks exit-trucks + 1 die]]

end
;;find the final destination
to-report find-final-destination
  let rndwheel random-float 1
  let finaldest ""
  ask min-one-of FinalDestinations [abs (finaldestiantion-ratio - rndwheel)]
    [set finaldest finaldestination-ID]
  report finaldest
end
;;find final destination path
to truck-find-final-path [destinationnodeid]
  let dnodeid destinationnodeid
  let denternode one-of nodes with [node-link-ID = dnodeid]
  let destinationnode ""
  let my-r current-location-node
  ask denternode
    [set destinationnode min-one-of nodes [distance myself]]
  ask current-location-node
    [set my-r nw:turtles-on-weighted-path-to destinationnode dismiles]
  ifelse my-r != false
    [set my-route my-r] ;if
    [let mlist [] ;else
    set mlist lput current-location-node mlist
    set my-route mlist]
end
to update-stations-connectors
  ask stations[
    set station-customers count trucks with [(status? = "Off-Duty")] in-radius 0.25
    ifelse station-capacity = 0
      [set station-usage-ratio 0]
      [set station-usage-ratio station-customers / station-capacity]
    ifelse station-usage-ratio > 0.5
      [set color red]
      [ifelse station-usage-ratio < 0.5
        [set color green]
        [set color yellow]]]
end

;;weibull distribution
to-report random-weibull [#scale #shape] ; goes scale then shape
  let xWei random-float 1
  let yWei (1 / #scale)
  let zWei (#shape * (-1 * ln(1 - xWei)))^ yWei
  report zwei
end

;;;;;;;;;;;;;;;Exporting truck results does not affect the simulation or post processing, but acts as a tool to double check truck agent behavior;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to Export-Trucks-CSV-0

  file-open "./results/truckresultsweek0.csv"
  ;file-print csv:to-row (list "Truck ID" "Truck Type" "On Duty Driving" "On Duty Not Driving" "Off Duty")
  ask trucks[
    file-print csv:to-row (list truck-ID profile-type Driving-Day NotDriving-Day OffDuty-Day)
    let results []
    set results lput truck-ID results
    set results lput profile-type results
    set results lput Driving-Day results
    set results lput NotDriving-Day results
    set results lput OffDuty-Day results
    file-print csv:to-row results
  ]
  file-flush
  file-close
end

;;modified this part from Dec.30 meeting
to Export-Trucks-CSV-1

  file-open "./results/truckresultsweek1.csv"
  ask trucks[
    ;file-print csv:to-row (list truck-ID profile-type Driving-Day NotDriving-Day OffDuty-Day)
    let results []
    set results lput truck-ID results
    set results lput profile-type results
    set results lput Driving-Day results
    set results lput NotDriving-Day results
    set results lput OffDuty-Day results
    file-print csv:to-row results
  ]
  file-flush
  file-close
end

to Export-Trucks-CSV-2

  file-open "./results/truckresultsweek2.csv"
  ask trucks[
    let results []
    set results lput truck-ID results
    set results lput profile-type results
    set results lput Driving-Day results
    set results lput NotDriving-Day results
    set results lput OffDuty-Day results
    file-print csv:to-row results]
  file-flush
  file-close
end

;;;;;;;;;;;;;;;;;;;;;;;;;exporting stations REQUIRED to receive recommendations- toggle switch located on Interface tab;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to export-stations-csv
  file-open "./results/StationsResults.csv"
;  file-print csv:to-row (list "CurrentTick" "Parking ID" "Number of Parked Trucks" "Number of Violated Trucks" "Parking Capacity" "Ratio")
  ask stations[
    set station-results []
    set station-results lput (word ticks) station-results
    set station-results lput (word station-name) station-results
    set station-results lput station-ID station-results
    set station-results lput (word station-customers) station-results
    set station-results lput (word station-violations) station-results
    set station-results lput (word station-capacity) station-results
    set station-results lput (word ((station-customers + station-violations) / station-capacity)) station-results
    file-print csv:to-row station-results]
  file-flush
  file-close
end
to violation-update
  ask trucks
  [
    ifelse status? = "Off-Duty" and count stations in-radius 0.25 with [station-usage-ratio <= 1] = 0
    [set violation-parking true]
    [set violation-parking false]
  ]
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;Post Processing of Simulation Output Starts Here;;;;;;;;;;;;;;;;;;;;;;;;;;
to post-process
  py:setup py:python2
  (py:run
    "import collections"
    "import csv"
    "data_file='./results/StationsResults.csv'"
    "parking_ids={}"
    "with open(data_file) as csvfile:"
    "  readCSV = csv.reader(csvfile, delimiter=',')"
    "  for row in readCSV:"
    "        if row[0] == 'CurrentTick':"
    "            continue"
    "        else:"
    "            if row[1] not in parking_ids:"
    "                parking_ids[row[1]]=[{row[0]:float(row[3])}]"
    "            else:"
    "                parking_ids[row[1]]. append({row[0]:float(row[3])})"
    "final_result={}"
    "for id in parking_ids:"
    "    final_result[id]={}"
    "    for i in range(1,25):"
    "        final_result[id][i]=0.0"
    "for id, data in parking_ids.items():"
    "    count = 0"
    "    for i in range(100):"
    "        if i%4==0:"
    "            count+=1"
    "            hour_sum=0"
    "            hour_sum+=list(data[i].values())[0]"
    "        else:"
    "            hour_sum += list(data[i].values())[0]"
    "        final_result[id][count]=hour_sum"
    "file_name = './results/Parking_hourly_result.csv'"
    "with open(file_name, 'w') as my_csv:"
    "    csv_writer = csv.writer(my_csv, delimiter=',')"
    "    csv_writer.writerow('P123456789')"
    "    for id,data in final_result.items():"
    "        d=[id]"
    "        data=collections.OrderedDict(sorted(data.items()))"
    "        for i in range(len(data.keys())):"
    "            d.append(data[i+1])"
    "        csv_writer.writerow(d)"
    "my_csv.close()"
    )
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to process-R
  r:clear
  r:clearLocal
  r:put "workdir" Working-Directory
  r:eval "setwd(workdir)"

  r:eval "library(knitr)"
  r:eval "library(rmarkdown)"
  r:eval "library(listviewer)"
  r:eval "library(Rglpk)"
  r:eval "library(dplyr)"
  r:eval "library(magrittr)"
  r:eval "library(ROI)"
  r:eval "library(ROI.plugin.glpk)"
  r:eval "library(ompr)"
  r:eval "library(ompr.roi)"
  r:eval "library(gtools)"
  r:eval "library(R.utils)"
  r:eval "library(reader)"
  r:eval "library(stringr)"

  r:put "M1" 176
  r:put "N1" 168

  r:put "TickMin" Tick-Minutes
  r:eval "Num_Tick_Per_Hours <- 60/TickMin"
  ;show r:get "Num_Tick_Per_Hours"
  r:put "Simulation_Day" Number-of-Days-to-Run-the-Model
  r:eval "Simulation_Hours <- Simulation_Day*24"
  ;show r:get "Simulation_Hours"
  r:eval "Simulation_Length <- Simulation_Hours*Num_Tick_Per_Hours"
  ;show r:get "Simulation_Length"

  r:eval "Distances <- read.delim('./DistanceFull.txt', sep ='', header= FALSE)"
  r:eval "CostsCapacities <- read.delim('./CostCapacity.txt', sep = '', header= FALSE)"
  r:eval "f <- CostsCapacities$V1"
  r:eval "f <- as.matrix(f)"
  r:eval "q <- CostsCapacities$V2"
  r:eval "q <- as.matrix(q)"

  r:eval "D <- read.csv('./results/StationsResults.csv', header =TRUE)"
  r:eval "Facilities_176 <- read.delim('./176Facilities.txt', sep = '\t', header= FALSE)"
  r:eval "Facilities_168 <- read.delim('./168Facilities.txt', sep = '\t', header= FALSE)"

  r:eval "D <- D[mixedorder(as.character(D$Parking.ID)),]"
  r:eval "D$ID <- seq.int(nrow(D))"
  r:eval "D$ID <- (D$ID %/%Simulation_Length)+1"
  r:eval "D$OverCapacity<- with(D, round(pmax(Ratio-1,0)*Parking.Capacity))"

  r:eval "h <- runif(N1, 0)"
  r:eval "d <- runif(N1, 0)"

  let k 1
  while [k <= 168]
  [ r:put "k" k
    r:eval "h[k] <- D %>% filter(ID==k) %>% select(OverCapacity) %>% summarise(x = max(OverCapacity))"
    r:eval "d[k] <- h[[k]]"
    set k k + 1]

  let j 1
  let ifAns true
  r:eval "d1 <- data.frame()[1,]"
  r:eval "Distances1 <- data.frame()[1:M1, ]"
  r:eval "NamesVec <- data.frame()[1,]"
  while [j <= 168]
    [ r:put "j" j
      set ifAns r:get "ifAns <- (d[j] != 0)"
      if ifAns = true [
        r:eval "Distances1 <- cbind(Distances1, Distances[,j])"
        r:eval "d1 <- cbind(d1, d[j])"
        r:eval "NamesVec <- cbind(NamesVec,j)"]
      set j j + 1]

  r:eval "NamesVec <- t(NamesVec)"
  r:eval "d1 <- t(d1)"
  r:eval "rownames(d1) <-c(NamesVec)"

  r:eval "rownames(Distances1)<-(1:M1)"
  r:eval "colnames(Distances1)<-c(NamesVec)"

  r:eval "N2<-as.numeric(ncol(Distances1))"
  r:eval "w<-function(i,j){ vapply(seq_along(i), function(k) Distances1[i[k], j[k]], numeric(1L))}"

  r:put "max_Distance" 40
  r:put "max_budget" Budget

  r:eval "sink('./logofcode.txt')"
  ;r:eval "source('./w.R')"
  r:eval "model <- MIPModel() %>% add_variable(y[i], i = 1:M1, type = 'binary') %>% add_variable(x[i,j], i = 1:M1, j=1:N2, type = 'continuous', lb = 0, ub =1 ) %>% add_variable(z[i,j], i = 1:M1, j=1:N2, type = 'binary') %>% set_objective(sum_expr(x[i,j]*d1[j], i = 1:M1, j=1:N2)/sum_expr(d1[j], j=1:N2), 'max') %>% add_constraint(z[i,j]*w(i,j) <= y[i] * max_Distance , j=1:N2, i = 1:M1) %>% add_constraint(x[i,j]-z[i,j] <=0.0  , i = 1:M1, j=1:N2) %>% add_constraint(sum_expr(f[i]*y[i], i = 1:M1)<= max_budget) %>% add_constraint(sum_expr(x[i, j], i = 1:M1) <= 1.0, j = 1:N2) %>% add_constraint(sum_expr(d1[j]*x[i, j], j = 1:N2)<= y[i] * q[i] , i = 1:M1) %>% add_constraint(y[k]+y[k+42]+y[k+84] <= 1.0, k = 1:42)"
  r:eval "result <- solve_model(model, with_ROI(solver = \"glpk\", verbose=TRUE, tm_limit = 1000*1800))"
  r:eval "sink()"

  r:eval "LogfileLines<-as.numeric(countLines('./logofcode.txt', chunkSize=5e+07))"
  r:eval "my_txt_ex2 <- n.readLines(paste('./logofcode.txt', sep = ''),header = FALSE, n = 1,  skip =LogfileLines-3 )"
  r:eval "startPercentage<-str_locate(my_txt_ex2,'%')[1,1]"
  r:eval "endPercentage<-str_locate(my_txt_ex2,'%')[1,2]"
  r:eval "OptimalityGap<-substring(my_txt_ex2, startPercentage-4, endPercentage)"
  r:eval "OptimalityGap3<-paste('near optimal with an optimality gap of', OptimalityGap)"


;  r:eval "matching <- result"
;  r:eval "soln.x <- get_solution(result, x[i,j])"
;  r:eval "soln.y <- get_solution(result, y[i])"
;  r:eval "lapply(matching$solution, head)"
;  r:eval "summary(matching)"

  let ifResult false
  set ifResult r:get "ifOpt3 <- (result$objective_value >0 & result$objective_value <1 & result[['status']] =='infeasible')"
  if ifResult = true
    [r:eval "result[['status']] <- OptimalityGap3"]
  let checkBudgErr true
  set checkBudgErr r:get "checkBudgErr <- (result[['status']] == 'infeasible' & max_budget>3000000)"
  if checkBudgErr = true [
    r:eval "result[['status']] <- 'opt'"
    output-print "Optimal solution exists. Reducing budget from $" output-print Budget
  ]

  let adjBudg r:get "adjBudg <- (result[['status']] == 'opt')"
  while [adjBudg = true][

    r:eval "rm(model)"
    r:eval "rm(result)"

    r:eval "max_budget = max_budget - 2000000"
    r:eval "model <- MIPModel() %>% add_variable(y[i], i = 1:M1, type = 'binary') %>% add_variable(x[i,j], i = 1:M1, j=1:N2, type = 'continuous', lb = 0, ub =1 ) %>% add_variable(z[i,j], i = 1:M1, j=1:N2, type = 'binary') %>% set_objective(sum_expr(x[i,j]*d1[j], i = 1:M1, j=1:N2)/sum_expr(d1[j], j=1:N2), 'max') %>% add_constraint(z[i,j]*w(i,j) <= y[i] * max_Distance , j=1:N2, i = 1:M1) %>% add_constraint(x[i,j]-z[i,j] <=0.0  , i = 1:M1, j=1:N2) %>% add_constraint(sum_expr(f[i]*y[i], i = 1:M1)<= max_budget) %>% add_constraint(sum_expr(x[i, j], i = 1:M1) <= 1.0, j = 1:N2) %>% add_constraint(sum_expr(d1[j]*x[i, j], j = 1:N2)<= y[i] * q[i] , i = 1:M1) %>% add_constraint(y[k]+y[k+42]+y[k+84] <= 1.0, k = 1:42)"
    r:eval "result <- solve_model(model, with_ROI(solver = \"glpk\", verbose=TRUE, tm_limit = 1000*1800))"
    r:eval "sink()"

    r:eval "LogfileLines<-as.numeric(countLines('./logofcode.txt', chunkSize=5e+07))"
    r:eval "my_txt_ex2 <- n.readLines(paste('./logofcode.txt', sep = ''),header = FALSE, n = 1,  skip =LogfileLines-3 )"
    r:eval "startPercentage<-str_locate(my_txt_ex2,'%')[1,1]"
    r:eval "endPercentage<-str_locate(my_txt_ex2,'%')[1,2]"
    r:eval "OptimalityGap<-substring(my_txt_ex2, startPercentage-4, endPercentage)"
    r:eval "OptimalityGap3<-paste('near optimal with an optimality gap of', OptimalityGap)"

    let ifResultLoop false
    set ifResultLoop r:get "ifOpt3Loop <- (result$objective_value >0 & result$objective_value <1 & result[['status']] =='infeasible')"
    if ifResultLoop = true
      [r:eval "result[['status']] <- OptimalityGap3"]
    let checkBudgErrLoop r:get "checkBudgErr <- (result[['status']] == 'infeasible' & max_budget>3000000)"
    if checkBudgErrLoop = true [
      r:eval "result[['status']] <- 'opt'"
    output-print "Optimal solution exists. Reducing budget from $" output-print r:get "max-budget"
    set adjBudg r:get "adjBudg <- (result[['status']] == 'opt')"
    ]

  ]

  r:eval "matching <- result"
  r:eval "lapply(matching$solution, head)"
  r:eval "summary(matching)"

  r:eval "soln.x <- get_solution(result, x[i,j])"
  ;show r:get "soln.x"
  r:eval "write.table(soln.x, file='./Xij.txt', sep='\t')"
  ;show "wrote table for soln.x"
  r:eval "soln.y <- get_solution(result, y[i])"
  r:eval "write.table(soln.y, file='./Yi.txt', sep='\t')"

  r:eval "DemandFacWithNames<-Facilities_168[Facilities_168$V1 %in% NamesVec,]"
  r:eval "DemandFacWithNames<-cbind(DemandFacWithNames, Temp_ID = seq(1:nrow(NamesVec)))"

  r:eval "soln.y <- cbind(soln.y, Facilities_176$V2)"
  r:eval "soln.y <- cbind(soln.y, Facilities_176$V3)"
  r:eval "soln.y <- rename(soln.y, 'ID' = 'Facilities_176$V2')"
  r:eval "soln.y <- rename(soln.y, 'Name' = 'Facilities_176$V3')"
  r:eval "soln.y$ID <- as.character(soln.y$ID)"
  r:eval "soln.y$Name <- as.character(soln.y$Name)"
  r:eval "soln.y <- cbind(soln.y, CostsCapacities$V1, CostsCapacities$V2)"

  r:eval "y1<-filter(soln.y, soln.y$i <  43  & soln.y$value==1)"
  r:eval "y2<-filter(soln.y, soln.y$i >= 43  & soln.y$i<=85 & soln.y$value==1)"
  r:eval "y3<-filter(soln.y, soln.y$i >= 85  & soln.y$i<=126 & soln.y$value==1)"
  r:eval "y4<-filter(soln.y, soln.y$i >= 127 & soln.y$value==1)"

  r:eval "ID_Supplies<-as.character(rep(Facilities_176$V2, times = N2))"
  r:eval "ID_Demands<-as.character(rep(DemandFacWithNames$V2, each = M1))"
  r:eval "Name_Supplies<-as.character(rep(Facilities_176$V3, times = N2))"
  r:eval "Name_Demands<-as.character(rep(DemandFacWithNames$V3, each = M1))"
  r:eval "soln.x<-cbind(soln.x, ID_Supplies, ID_Demands, Name_Supplies, Name_Demands)"

  r:eval "x1<-filter(soln.x, soln.x$i <  43  & soln.x$value!=0)"
  r:eval "x2<-filter(soln.x, soln.x$i >= 43  & soln.x$i<=85 & soln.x$value!=0)"
  r:eval "x3<-filter(soln.x, soln.x$i >= 85  & soln.x$i<=126 & soln.x$value!=0)"
  r:eval "x4<-filter(soln.x, soln.x$i >= 127 & soln.x$value!=0)"

  ;0 to report based on name, 1 based on ID
  r:put "Report" 0

  r:eval "source('./myFunction.R')"
  r:eval "myFunction(result, Report, f, soln.y, x1, x2, x3, x4, y1, y2, y3, y4)"

  ;draw-new-stations

  file-open "./results/recommendations.txt"
  while [not file-at-end?]
  [output-print file-read-line]
  file-close

  draw-new-stations
end

to draw-new-stations

  file-open "./results/recommendedStationIDs.txt" ;recommendedStationIDs.txt
  let stns gis:load-dataset "NewStations.shp"
  gis:set-world-envelope gis:envelope-of stns
  foreach gis:feature-list-of stns[
    vector-feature ->
    ask patches gis:intersecting vector-feature[
      sprout-newStations 1[
      set newStation-ID gis:property-value vector-feature "IDNumber"
      set newStation-type gis:property-value vector-feature "FacilityTy"
        ifelse newStation-type = "Private" [set shape "parkingprivate"][set shape "parkingpublic"]
        set color 8
        set size 1
      set hidden? true]
    ]
  ]
  while [not file-at-end?]
  [ let currentLine file-read-line
    ask newStations[
      if newStation-ID = currentLine [set color pink set size 5 set hidden? false]
    ]
  ]
  file-close
end
@#$#@#$#@
GRAPHICS-WINDOW
198
42
1049
654
-1
-1
3.0
1
10
1
1
1
0
0
0
1
-140
140
-100
100
0
0
1
ticks
30.0

BUTTON
0
56
64
89
NIL
Setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
67
56
130
89
NIL
Start
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

INPUTBOX
1054
75
1207
135
Highway-Speed
70.0
1
0
Number

PLOT
1473
152
1885
340
Existing Trucks Status
Ticks
Number of Trucks 
0.0
100.0
0.0
60.0
false
true
"" ""
PENS
"On Duty Driving" 1.0 0 -15040220 true "" "if ticks > 0 [plot count trucks with [status? = \"On-Duty-Driving\" and profile-type = \"existingtruck\"]]"
"On Duty Not Driving" 1.0 0 -2674135 true "" "if ticks > 0 [plot count trucks with [status? = \"On-Duty-Not-Driving\" and profile-type = \"existingtruck\"]]"
"Off Duty" 1.0 0 -16777216 true "" "if ticks > 0 [plot count trucks with [status? = \"Off-Duty\" and profile-type = \"existingtruck\"]]"

MONITOR
710
658
865
703
Total Trucks
Count trucks
17
1
11

MONITOR
880
658
1034
703
On Duty Trucks
count trucks with [status? = \"On-Duty-Driving\"]
17
1
11

CHOOSER
0
129
196
174
Tick-Minutes
Tick-Minutes
1 10 20 30 60
1

MONITOR
367
658
522
703
Current Hour
current-hour + 1
17
1
11

MONITOR
540
658
695
703
Current Day
current-day + 1
17
1
11

MONITOR
1219
659
1374
704
Existing Trucks
count trucks with [profile-type = \"existingtruck\"]
17
1
11

BUTTON
133
56
196
89
Stop
stop-model
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

TEXTBOX
205
10
1039
44
Truck Symbol: BLUE trucks are entering trucks, ORANGE trucks are existing trucks. Off-duty trucks are WHITE.\nParking Facility Symbol, P: rectangular stops are privately owned, circular stops are publicly owned.(Full: Red, Half-full: Yellow, and Empty: Green)
12
1.0
1

MONITOR
1049
659
1204
704
Off Duty Trucks
count trucks with [status? = \"Off-Duty\"]
17
1
11

INPUTBOX
0
197
197
257
Number-of-Days-to-Run-the-Model
14.0
1
0
Number

MONITOR
1390
660
1545
705
Entering Trucks
int current-hour-enter-trucks
17
1
11

SLIDER
1383
101
1582
134
Risk-Taker-Ratio
Risk-Taker-Ratio
0
100
20.0
10
1
%
HORIZONTAL

PLOT
197
1089
710
1276
Average Facility Usage
Ticks
Facility Usage Ratio
0.0
10.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -2674135 true "" "if ticks > 0 [plot mean [station-usage-ratio] of stations]"

PLOT
1056
347
1470
534
Exited Trucks per Tick
Ticks
Number of Trucks
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" "if ticks > 0 [plot exited-this-tick]"

PLOT
1055
153
1470
340
Entering Trucks Status
Ticks
Number of Trucks
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"On Duty Driving" 1.0 0 -12087248 true "" "if ticks > 0 [plot count trucks with [status? = \"On-Duty-Driving\" and profile-type = \"enteringtruck\"]]"
"On Duty Not Driving" 1.0 0 -2674135 true "" "if ticks > 0 [plot count trucks with [status? = \"On-Duty-Not-Driving\" and profile-type = \"enteringtruck\"]]"
"Off Duty" 1.0 0 -16777216 true "" "if ticks > 0 [plot count trucks with [status? = \"Off-Duty\" and profile-type = \"enteringtruck\"]]"

PLOT
711
706
1224
893
Violate Parking
Ticks
Number of Trucks (%)
0.0
20.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -2674135 true "" "if ticks > 0 [\nlet vi count trucks with [violation-parking = true]\nplot (vi / count trucks) * 100] ;with [status? = \"Off-Duty\"]]"
"day" 1.0 0 -16777216 true "" "if ticks mod 144 = 0 [\n  plot-pen-up\n  plotxy ticks 0\n  plot-pen-down\n  plotxy ticks plot-y-max]"

PLOT
712
897
1224
1084
Violate HOS
Ticks
Number of Trucks (%)
0.0
10.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -2674135 true "" "if ticks > 0 [\nlet vi count trucks with [violation-HOS = true]\nplot ((vi / count trucks) * 100)]"
"pen-1" 1.0 0 -16777216 true "" "if ticks mod 144 = 0 [\n  plot-pen-up\n  plotxy ticks 0\n  plot-pen-down\n  plotxy ticks plot-y-max]"

MONITOR
196
658
351
703
Total Time Passed (hour)
precision time-now 2
17
1
11

PLOT
1473
347
1886
534
Risk Taking Trucks
Ticks
Number of Trucks
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Risk Taker" 1.0 0 -5298144 true "" "if ticks > 1 [plot count trucks with [risk-type = \"risk-taker\"]]"
"Risk Averse" 1.0 0 -14439633 true "" "if ticks > 1 [plot count trucks with [risk-type = \"risk-averse\"]]"

MONITOR
1560
659
1715
704
Risk Takers
count trucks with [risk-type = \"risk-taker\"]
17
1
11

MONITOR
1731
659
1886
704
Risk Aversers
count trucks with [risk-type = \"risk-averse\"]
17
1
11

SWITCH
0
304
196
337
Save-Stations-Usage-Results
Save-Stations-Usage-Results
0
1
-1000

TEXTBOX
3
277
153
295
Export Results\n
13
101.0
0

TEXTBOX
4
40
154
58
Model Run
12
101.0
1

TEXTBOX
4
110
154
128
Select Tick Minutes\n
12
0.0
1

TEXTBOX
1059
57
1209
75
Set Speed (MPH)
13
0.0
1

TEXTBOX
1385
83
1552
115
Select Risk Taker Ratio
13
0.0
1

TEXTBOX
4
181
154
199
Simulation Duration
12
0.0
1

TEXTBOX
1410
541
1589
575
Simulation Runtime Results
14
0.0
1

TEXTBOX
49
884
190
908
Parking Usage Results
14
0.0
1

SWITCH
0
349
194
382
Save-Truck-Results
Save-Truck-Results
0
1
-1000

PLOT
197
705
708
892
Max Facility Usage
Ticks
Facility Usage Ratio
0.0
10.0
0.0
1.0
true
false
"" ""
PENS
"max" 1.0 2 -2674135 true "" "if ticks >= 1 [plot max [station-usage-ratio] of stations]"

SWITCH
1
395
195
428
Summarize-Usage-Data
Summarize-Usage-Data
0
1
-1000

MONITOR
1057
534
1200
579
Total Exited Trucks
exit-trucks
17
1
11

PLOT
197
896
709
1083
Total Parking Usage
Ticks
Percentage
0.0
10.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "if ticks > 0 \n  [let offduty count trucks with [status? = \"Off-Duty\"] \n   plot (offduty / 6887) * 100]"

INPUTBOX
1215
75
1375
135
DailyDrivingHours
11.0
1
0
Number

OUTPUT
1226
733
1886
1110
11

TEXTBOX
1494
712
1644
730
Recommendations
14
0.0
1

INPUTBOX
3
480
196
550
Working-Directory
C:\\\\Users\\\\arlacrue\\\\Desktop\\\\ParkSim
1
0
String

TEXTBOX
1214
38
1364
98
Set Daily Driving Hours (HOS, for daily max)
14
0.0
1

SLIDER
3
585
195
618
Budget
Budget
0
30000000
2.0E7
1000000
1
$
HORIZONTAL

BUTTON
2
655
194
688
Run Optimization Model
process-R
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

TEXTBOX
12
625
199
667
Click to receive recommendations (must run simulation at least once)
11
0.0
1

TEXTBOX
8
552
192
597
Set Budget for Recommendations (in millions)
12
0.0
1

TEXTBOX
6
445
188
490
Set Working Directory (required to receive recommendations)
12
0.0
1

@#$#@#$#@
<img src = "./photos/UA_Logo_Horizontal.jpg" alt="University of Arkansas Logo" height = "170" />

# Arkansas Commercial Truck Safety and Education Program (ACTSEP)

## DISCLAIMER

The model, ParkSIM, is a tool to estimate truck parking demand based on assumed usage patterns, facility capacities, and driver behavior. The results of ParkSIM do not necessarily reflect observed truck parking usage patterns.

## WHAT IS IT?

The purpose of this simulation is to provide recommendations for possible parking facility creation and expansions by simulating the movement of trucks through the state of Arkansas.

## SETUP: IMPORTANT! PLEASE READ

**_This simulation requires R version 3 or later to be installed on your computer, see [manual](./ParkSimImplementationReportACTSEP10012020.pdf) for further instructions._**

**Before** starting a new simulation and clicking "Setup" for the first time, you must set the Working Directory for the project. The Working Directory tells NetLogo where to look for the files needed to complete the various calculations used in the simulation. 

To set your working directory, navigate to the file folder that holds the ParkSIM files. Click the address bar and copy the filepath highlighted:
![address bar](./photos/folderoverview.png)

Next, back in the ParkSIM program, paste the copied filepath into the box labeled "Working-Directory" found in the **Interface** tab.
![working directory](./photos/workingdirectory.png)
_For Windows Users_: By default, Windows uses "\\" to separate locations in a filepath. To properly set your Working Directory, change each "\\" to "\\\\" for the simulation to read and set your Working Directory. When finished, the Working Directory you set will look something like this: 
C:\\Users\\username\\Desktop\\ParkSim

To generate recommendations, users must have the **"Save-Stations-Usage-Results"** and **"Summarize-Usage-Data"** toggles flipped to the "ON" position.

Additionally, the "Interface" tab has several different fields that can be used to alter the simulation.

  * Number-of-Days-to-Run-the-Model: How many days the simulation will run before generating recommendations
  * Tick-Minutes: How many minutes are represented by each "tick" of the simulation. For example, setting this value to **10** means every six ticks of the simulation equals one hour.
  * Highway-Speed: How fast trucks will move, in miles per hour.
  * DailyDrivingHours: How many hours a truck can drive each day before taking a long (8+ hour) rest.
  * Risk-Takers: Used as a percent. Trucks can be "risk taking" or "risk averse." Risk Takers will wait longer to begin searching for a parking location and therefore are more likely to violate their hours of service (HOS) rules, and more likely to park illegally.
  * Budget: Used when generating recommendations to determine how many additional spots or facilities can be built (Cost estimations for various types of facilities can be found in the [user manual](./ParkSimImplementationReportACTSEP10012020.pdf).

## THINGS TO TRY
Once you have run the simulation once, you can use the "Run Optimization Model" button on the **Interface** tab to calculate recommendations without having to re-run the entire simulation. This is a handy tool if you want to test the effects of different budgets on the recommendations generated.

## HOW IT WORKS

Using historic truck profiles, truck agents are created and begin to traverse the map. Each truck follows a set of service rules, limiting how far a truck can travel before being forced to park and rest. Truck agents can be either "risk taking" or "risk averse," a value that determines how far in advance a truck will begin looking for a parking spot before violating the maximum number of hours it is allowed to drive. A truck that drives over the maximum number of hours allowed will either continue to drive until it finds a parking spot (violating hours of service regulations) or will park illegally. After resting, a truck will begin driving again until it either reaches another hours of service limit or leaves the state.

## CREDITS AND REFERENCES

For more information, see the [ParkSIM User Manual](./ParkSimImplementationReportACTSEP10012020.pdf).

Created By:

**Dr. Sarah Vavrik Hernandez**
_PI, College of Engineering, Department of Civil Engineering_

**Dr. Sarah G. Nurre**
_PI, College of Engineering, Department of Industrial Engineering_

**Marjan Asadinia**
_Postdoctoral Fellow, Industrial and Civil Engineering Departments_

**Taslima Akter, Research Assistant**
**Amin Asadi, Research Assistant**
**Sharif Mahmud, Research Assistant**
**Annabelle LaCrue, Research Assistant**

_last updated, Oct. 2020_
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

parking
false
1
Rectangle -7500403 true false 75 60 225 240
Rectangle -16777216 true false 90 75 210 225
Polygon -7500403 true false 105 105 105 210 120 210 120 150 165 150 195 150 195 120 195 90 105 90
Rectangle -16777216 true false 120 105 180 135

parkingprivate
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240
Polygon -7500403 true true 90 90 90 210 105 210 105 150 210 150 210 90
Rectangle -16777216 true false 105 105 195 135

parkingpublic
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Polygon -7500403 true true 90 90 90 240 105 240 105 165 210 165 210 90
Rectangle -16777216 true false 105 105 195 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.1.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
